==> satellite-system-2016.1.7-sources.jar: com/xebialabs/deployit/plugin/satellite/SyncExtensionsStep.scala <==
package com.xebialabs.deployit.plugin.satellite

import java.io.{File, ObjectInputStream}
import java.net.URI
import java.nio.file

import akka.actor.{ActorRef, ActorSystem}
import akka.pattern._
import akka.util.Timeout
import com.xebialabs.deployit.engine.tasker.satellite.ActorLocator
import com.xebialabs.deployit.engine.tasker.{TaskExecutionContext, TaskSpecification}
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, Step, StepExitCode}
import com.xebialabs.deployit.plugin.satellite.UploadTaskSupervisor.Protocol.{FileToUpload, UploadConfig, UploadFilesTask, UploadTaskFinished}
import com.xebialabs.deployit.plugin.satellite.extension.{ExtensionsLocator, FileSystemExtensionsLocator}
import com.xebialabs.satellite.future.AwaitForever
import com.xebialabs.satellite.protocol.{Directories, Paths, ReplaceExtensionsFromTask}
import com.xebialabs.xlplatform.satellite.Satellite

import scala.beans.BeanProperty
import scala.concurrent.duration._

case class SyncExtensionsStep(actorLocator: ActorLocator, satelliteHostname: String, @BeanProperty description: String,
                             extensionsLocator: ExtensionsLocator)
                             (implicit @transient var satelliteCommunicatorSystem: ActorSystem, @transient var uploader: ActorRef)
  extends Step with AwaitForever with SatellitePluginsSugar {

  override def execute(ctx: ExecutionContext): StepExitCode = {
    ctx.logOutput(s"Connecting to $actorLocator")

    val deltas = calculateExtensionsDelta(extensionsLocator, ctx)
    sendFilesToSatellite(deltas, ctx) match {
      case None =>
        ctx.logOutput("Satellite is up-to-date.")
        StepExitCode.SUCCESS
      case Some(UploadTaskFinished(_, Some(error))) =>
        ctx.logError(s"Error while sending extensions to the satellite. ($error)")
        StepExitCode.FAIL
      case Some(UploadTaskFinished(filesUploaded, None)) =>
        replaceExtensionsIfNeeded(filesUploaded, ctx)
    }
  }

  private def sendFilesToSatellite(deltas: InstalledExtensionsDelta, ctx: ExecutionContext) = {
    if (!deltas.empty) {
      val directories = List(
        Directories.PLUGINS -> deltas.plugins,
        Directories.HOTFIXES -> deltas.hotfix,
        Directories.EXT -> deltas.ext
      )

      logIfEmpty(directories, ctx)
      val files = filesToUpload(directories)
      val uploadConfig = generateUploadConfig(ctx)
      val uploadTask = UploadFilesTask(uploadConfig, files)

      implicit val timeout = Timeout(1.day)
      implicit val executionContext = satelliteCommunicatorSystem.dispatcher

      Some(blockOrThrow((uploader ? uploadTask).mapTo[UploadTaskFinished]))
    } else None
  }

  private def logIfEmpty(directories: List[(String, Set[URI])], ctx: ExecutionContext) {
    directories.foreach { case (dir, uris) =>
      if (uris.isEmpty) {
        ctx.logOutput(s"$dir is up-to-date")
      }
    }
  }

  private def filesToUpload(directories: List[(String, Set[URI])]) = {
    directories.foldLeft(List[FileToUpload]()) { case (accumulator, (_, uris)) =>
        accumulator ::: uris.map { case uri =>
          val pluginFile = PluginLocalFile(new File(uri))

          val basePath = file.Paths.get(new java.io.File(".").getCanonicalPath)
          val filePath = file.Paths.get(pluginFile.getPath)
          val relativePath = basePath.relativize(filePath).getParent
          FileToUpload(relativePath.toString, pluginFile)
        }.toList
    }
  }

  private def generateUploadConfig(ctx: ExecutionContext) = {
    val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
    UploadConfig(task.getId, ctx, actorLocator, satelliteHostname)
  }

  private def replaceExtensionsIfNeeded(filesUploaded: Long, ctx: ExecutionContext) = {
    if(filesUploaded > 0) {
      ctx.logOutput("Replacing extensions")
      ctx.setAttribute("restartRequired", true)
      implicit val timeout = Timeout(1.day)
      implicit val executionContext = satelliteCommunicatorSystem.dispatcher
      val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
      val satellitePluginManagement = actorLocator.locate(Paths.pluginManagement)
      blockOrThrow(satellitePluginManagement ? ReplaceExtensionsFromTask(task.getId))
    } else {
      ctx.logOutput("Replacing extensions not necessary")
    }
    StepExitCode.SUCCESS
  }

  @BeanProperty val order = Step.DEFAULT_ORDER

  private def readObject(in: ObjectInputStream): Unit = {
    in.defaultReadObject()
    satelliteCommunicatorSystem = SatelliteCommunicatorSystem.actorSystem
    uploader = SatelliteCommunicatorSystem.uploader
  }

}

object SyncExtensionsStep {

  def apply(satellite: Satellite)(implicit satelliteCommunicatorSystem: ActorSystem, uploader: ActorRef): SyncExtensionsStep = {
    new SyncExtensionsStep(
      ActorLocator(satellite), satellite.getAddress,
      s"Synchronise extensions on satellite ${satellite.getName}",
      FileSystemExtensionsLocator
    )
  }

}

==> satellite-system-2017.1.1-sources.jar: com/xebialabs/deployit/plugin/satellite/SyncExtensionsStep.scala <==
package com.xebialabs.deployit.plugin.satellite

import java.io.{File, ObjectInputStream}
import java.net.URI
import java.nio.file

import akka.actor.{ActorRef, ActorSystem}
import akka.pattern._
import akka.util.Timeout
import com.xebialabs.deployit.engine.tasker.satellite.ActorLocator
import com.xebialabs.deployit.engine.tasker.{TaskExecutionContext, TaskSpecification}
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, Step, StepExitCode}
import com.xebialabs.deployit.plugin.satellite.UploadTaskSupervisor.Protocol.{FileToUpload, UploadConfig, UploadFilesTask, UploadTaskFinished}
import com.xebialabs.deployit.plugin.satellite.extension.{ExtensionsLocator, FileSystemExtensionsLocator}
import com.xebialabs.satellite.future.AwaitForever
import com.xebialabs.satellite.protocol.{Directories, Paths, ReplaceExtensionsFromTask}
import com.xebialabs.xlplatform.satellite.Satellite

import scala.beans.BeanProperty
import scala.concurrent.duration._

case class SyncExtensionsStep(actorLocator: ActorLocator, satelliteHostname: String, @BeanProperty description: String,
                              extensionsLocator: ExtensionsLocator, satelliteAddress: String)
                             (implicit @transient var satelliteCommunicatorSystem: ActorSystem, @transient var uploader: ActorRef)
  extends Step with AwaitForever with SatellitePluginsSugar {

  override def execute(ctx: ExecutionContext): StepExitCode = {
    ctx.logOutput(s"Connecting to satellite at $satelliteAddress")

    ctx.logOutput(s"Calculating which plugins need to be updated")
    val deltas = calculateExtensionsDelta(extensionsLocator, ctx)
    sendFilesToSatellite(deltas, ctx) match {
      case None =>
        ctx.logOutput("Satellite is up-to-date.")
        StepExitCode.SUCCESS
      case Some(UploadTaskFinished(_, Some(error))) =>
        ctx.logError(s"Error while sending extensions to the satellite. ($error)")
        StepExitCode.FAIL
      case Some(UploadTaskFinished(filesUploaded, None)) =>
        replaceExtensionsIfNeeded(filesUploaded, ctx)
    }
  }

  private def sendFilesToSatellite(deltas: InstalledExtensionsDelta, ctx: ExecutionContext) = {
    if (!deltas.empty) {
      val directories = List(
        Directories.PLUGINS -> deltas.plugins,
        Directories.HOTFIXES -> deltas.hotfix,
        Directories.EXT -> deltas.ext
      )

      logIfDirNeedsToBeSynced(directories, ctx)
      val files = filesToUpload(directories)
      val uploadConfig = generateUploadConfig(ctx)
      val uploadTask = UploadFilesTask(uploadConfig, files)

      implicit val timeout = Timeout(1.day)
      implicit val executionContext = satelliteCommunicatorSystem.dispatcher

      Some(blockOrThrow((uploader ? uploadTask).mapTo[UploadTaskFinished]))
    } else None
  }

  private def logIfDirNeedsToBeSynced(directories: List[(String, Set[URI])], ctx: ExecutionContext) {
    directories.foreach { case (dir, uris) =>
      if (uris.isEmpty) {
        ctx.logOutput(s"$dir is up-to-date")
      } else {
        ctx.logOutput(s"$dir needs to be synced")
      }
    }
  }

  private def filesToUpload(directories: List[(String, Set[URI])]) = {
    directories.foldLeft(List[FileToUpload]()) { case (accumulator, (_, uris)) =>
      accumulator ::: uris.map { case uri =>
        val pluginFile = PluginLocalFile(new File(uri))

        val basePath = file.Paths.get(new java.io.File(".").getCanonicalPath)
        val filePath = file.Paths.get(pluginFile.getPath)
        val relativePath = basePath.relativize(filePath).getParent
        FileToUpload(relativePath.toString, pluginFile)
      }.toList
    }
  }

  private def generateUploadConfig(ctx: ExecutionContext) = {
    val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
    UploadConfig(task.getId, ctx, actorLocator, satelliteHostname)
  }

  private def replaceExtensionsIfNeeded(filesUploaded: Long, ctx: ExecutionContext) = {
    if (filesUploaded > 0) {
      ctx.logOutput("Replacing existing satellite extensions with the uploaded ones. Satellite will be restarted.")
      ctx.setAttribute("restartRequired", true)
      implicit val timeout = Timeout(1.day)
      implicit val executionContext = satelliteCommunicatorSystem.dispatcher
      val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
      val satellitePluginManagement = actorLocator.locate(Paths.pluginManagement)
      blockOrThrow(satellitePluginManagement ? ReplaceExtensionsFromTask(task.getId))
    } else {
      ctx.logOutput("Replacing extensions not necessary")
    }
    StepExitCode.SUCCESS
  }

  @BeanProperty val order = Step.DEFAULT_ORDER

  private def readObject(in: ObjectInputStream): Unit = {
    in.defaultReadObject()
    satelliteCommunicatorSystem = SatelliteCommunicatorSystem.actorSystem
    uploader = SatelliteCommunicatorSystem.uploader
  }

}

object SyncExtensionsStep extends SatelliteAddress {

  def apply(satellite: Satellite)(implicit satelliteCommunicatorSystem: ActorSystem, uploader: ActorRef): SyncExtensionsStep = {
    new SyncExtensionsStep(
      ActorLocator(satellite), satellite.getAddress,
      s"Synchronise extensions on satellite ${satellite.getName}",
      FileSystemExtensionsLocator, satelliteAddress(satellite)
    )
  }

}
